@@ -398,6 +398,9 @@ BUCKET_STORAGE_CLASS_STANDARD = 'Standard'
 BUCKET_STORAGE_CLASS_IA = 'IA'
 BUCKET_STORAGE_CLASS_ARCHIVE = 'Archive'
 
+BUCKET_DATA_REDUNDANCY_TYPE_LRS = "LRS"
+BUCKET_DATA_REDUNDANCY_TYPE_ZRS = "ZRS"
+
 REDIRECT_TYPE_MIRROR = 'Mirror'
 REDIRECT_TYPE_EXTERNAL = 'External'
 REDIRECT_TYPE_INTERNAL = 'Internal'
@@ -440,8 +443,9 @@ class GetBucketLoggingResult(RequestResult, BucketLogging):
 
 
 class BucketCreateConfig(object):
-    def __init__(self, storage_class):
+    def __init__(self, storage_class, data_redundancy_type=None):
         self.storage_class = storage_class
+        self.data_redundancy_type = data_redundancy_type
 
 
 class BucketStat(object):
@@ -464,8 +468,8 @@ class Owner(object):
 
 class BucketInfo(object):
     def __init__(self, name=None, owner=None, location=None, storage_class=None, intranet_endpoint=None,
-                 extranet_endpoint=None, creation_date=None, acl=None, bucket_encryption_rule=None,
-                 versioning_status=None):
+                 extranet_endpoint=None, creation_date=None, acl=None, data_redundancy_type=None, comment=None,
+                 bucket_encryption_rule=None, versioning_status=None):
         self.name = name
         self.owner = owner
         self.location = location
@@ -474,6 +478,8 @@ class BucketInfo(object):
         self.extranet_endpoint = extranet_endpoint
         self.creation_date = creation_date
         self.acl = acl
+        self.data_redundancy_type = data_redundancy_type
+        self.comment = comment
 
         self.bucket_encryption_rule = bucket_encryption_rule
         self.versioning_status = versioning_status
@@ -770,13 +776,19 @@ class LifecycleExpiration(object):
     """过期删除操作。
 
     :param days: 表示在文件修改后过了这么多天，就会匹配规则，从而被删除
-    :param date: 表示在该日期之后，规则就一直生效。即每天都会对符合前缀的文件执行删除操作（如，删除），而不管文件是什么时候生成的。
-        *不建议使用*
-    :param created_before_date: delete files if their last modified time earlier than created_before_date
+    :type days: int
 
+    :param date: 表示在该日期之后，规则就一直生效。即每天都会对符合前缀的文件执行删除操作（如，删除），而不管文件是什么时候生成的。*不建议使用*
     :type date: `datetime.date`
+
+    :param created_before_date: delete files if their last modified time earlier than created_before_date
+    :type created_before_date: `datetime.date`
+
+    :param expired_detete_marker: 真实文件删除之后是否自动移除删除标记，适用于多版本场景。
+    :param expired_detete_marker: bool
+
     """
-    def __init__(self, days=None, date=None, created_before_date=None):
+    def __init__(self, days=None, date=None, created_before_date=None, expired_detete_marker=None):
         not_none_fields = 0
         if days is not None:
             not_none_fields += 1
@@ -784,13 +796,16 @@ class LifecycleExpiration(object):
             not_none_fields += 1
         if created_before_date is not None:
             not_none_fields += 1
+        if expired_detete_marker is not None:
+            not_none_fields += 1
 
         if not_none_fields > 1:
-            raise ClientError('More than one field(days, date and created_before_date) has been specified')
+            raise ClientError('More than one field(days, date and created_before_date, expired_detete_marker) has been specified')
 
         self.days = days
         self.date = date
         self.created_before_date = created_before_date
+        self.expired_detete_marker = expired_detete_marker
 
 
 class AbortMultipartUpload(object):
@@ -824,18 +839,52 @@ class StorageTransition(object):
         self.storage_class = storage_class
 
 
+class NoncurrentVersionExpiration(object):
+    """OSS何时将非当前版本的object删除
+
+    :param noncurrent_days: 指定多少天之后删除
+    :type noncurrent_days: int
+    """
+    def __init__(self, noncurrent_days):
+        self.noncurrent_days = noncurrent_days
+
+
+class NoncurrentVersionStorageTransition(object):
+    """生命周期内，OSS何时将指定Object的非当前版本转储为IA或者Archive存储类型。
+
+    :param noncurrent_days: 多少天之后转存储
+    :type noncurrent_days: int
+    """
+    def __init__(self, noncurrent_days, storage_class):
+        self.noncurrent_days = noncurrent_days
+        self.storage_class = storage_class
+
+
 class LifecycleRule(object):
     """生命周期规则。
 
     :param id: 规则名
+    :type id: str
+
     :param prefix: 只有文件名匹配该前缀的文件才适用本规则
+    :type prefix: str
+
     :param expiration: 过期删除操作。
     :type expiration: :class:`LifecycleExpiration`
+
     :param status: 启用还是禁止该规则。可选值为 `LifecycleRule.ENABLED` 或 `LifecycleRule.DISABLED`
+
     :param storage_transitions: 存储类型转换规则
-    :type storage_transitions: :class:`StorageTransition`
+    :type storage_transitions: list of class:`StorageTransition <oss2.models.StorageTransition>`
+
     :param tagging: object tagging 规则
-    :type tagging: :class:`Tagging`
+    :type tagging: :class:`Tagging <oss2.models.StorageTransition>`
+    
+    :param noncurrent_version_expiration: 指定Object非当前版本生命周期规则的过期属性。适用于多版本场景。
+    :type noncurrent_version_expiration class:`NoncurrentVersionExpiration <oss2.models.NoncurrentVersionExpiration>`
+
+    :param noncurrent_version_sotrage_transitions: 在有效生命周期中，OSS何时将指定Object的非当前版本转储为IA或者Archive存储类型，适用于多版本场景。
+    :type noncurrent_version_sotrage_transitions: list of class:`NoncurrentVersionStorageTransition <oss2.models.NoncurrentVersionStorageTransition>`
     """
 
     ENABLED = 'Enabled'
@@ -844,7 +893,9 @@ class LifecycleRule(object):
     def __init__(self, id, prefix,
                  status=ENABLED, expiration=None,
                  abort_multipart_upload=None,
-                 storage_transitions=None, tagging=None):
+                 storage_transitions=None, tagging=None,
+                 noncurrent_version_expiration=None,
+                 noncurrent_version_sotrage_transitions=None):
         self.id = id
         self.prefix = prefix
         self.status = status
@@ -852,13 +903,15 @@ class LifecycleRule(object):
         self.abort_multipart_upload = abort_multipart_upload
         self.storage_transitions = storage_transitions
         self.tagging = tagging
+        self.noncurrent_version_expiration = noncurrent_version_expiration
+        self.noncurrent_version_sotrage_transitions = noncurrent_version_sotrage_transitions
 
 
 class BucketLifecycle(object):
     """Bucket的生命周期配置。
 
     :param rules: 规则列表，
-    :type rules: list of :class:`LifecycleRule`
+    :type rules: list of :class:`LifecycleRule <oss2.models.LifecycleRule>`
     """
     def __init__(self, rules=None):
         self.rules = rules or []
@@ -1146,6 +1199,10 @@ class GetLiveChannelHistoryResult(RequestResult, LiveChannelHistory):
         RequestResult.__init__(self, resp)
         LiveChannelHistory.__init__(self)
 
+class GetVodPlaylistResult(RequestResult):
+    def __init__(self, resp):
+        RequestResult.__init__(self, resp)
+        self.playlist = to_string(resp.read())
 
 class ProcessObjectResult(RequestResult):
     def __init__(self, resp):
@@ -1324,4 +1381,136 @@ class GetBucketPolicyResult(RequestResult):
 class GetBucketRequestPaymentResult(RequestResult):
     def __init__(self, resp):
         RequestResult.__init__(self, resp)
-        self.payer = ''
\ No newline at end of file
+        self.payer = ''
+
+class BucketQosInfo(object):
+    """bucket的Qos信息    
+    :以下参数如果设置为0则表示完全禁止指定类型的访问，如果为-1则表示不单独限制
+
+    :param total_upload_bw: 总上传带宽, 单位Gbps
+    :type total_upload_bw: int
+
+    :param intranet_upload_bw: 内网上传带宽, 单位Gbps
+    :type intranet_upload_bw: int
+
+    :param extranet_upload_bw: 外网上传带宽, 单位Gbps
+    :type extranet_upload_bw: int
+
+    :param total_download_bw: 总下载带宽, 单位Gbps
+    :type total_download_bw: int
+
+    :param intranet_download_bw: 内外下载带宽, 单位Gbps
+    :type intranet_download_bw: int
+
+    :param extranet_download_bw: 外网下载带宽, 单位Gbps
+    :type extranet_download_bw: int
+
+    :param total_qps: 总qps, 单位请求数/s
+    :type total_qps: int
+
+    :param intranet_qps: 内网访问qps, 单位请求数/s
+    :type intranet_qps: int
+
+    :param extranet_qps: 外网访问qps, 单位请求数/s
+    :type extranet_qps: int
+    """
+    def __init__(self,
+            total_upload_bw = None,
+            intranet_upload_bw = None,
+            extranet_upload_bw = None,
+            total_download_bw = None,
+            intranet_download_bw = None,
+            extranet_download_bw = None,
+            total_qps = None,
+            intranet_qps = None,
+            extranet_qps = None):
+
+        self.total_upload_bw = total_upload_bw
+        self.intranet_upload_bw = intranet_upload_bw
+        self.extranet_upload_bw = extranet_upload_bw
+        self.total_download_bw = total_download_bw
+        self.intranet_download_bw = intranet_download_bw
+        self.extranet_download_bw = extranet_download_bw
+        self.total_qps = total_qps
+        self.intranet_qps = intranet_qps
+        self.extranet_qps = extranet_qps
+
+class UserQosInfo(object):
+    """User的Qos信息    
+
+    :param region: 查询的qos配置生效的区域
+    :type region: str
+
+    :以下参数如果为0则表示完全禁止指定类型的访问，如果为-1表示不单独限制
+
+    :param total_upload_bw: 总上传带宽, 单位Gbps
+    :type total_upload_bw: int
+
+    :param intranet_upload_bw: 内网上传带宽, 单位:Gbps
+    :type intranet_upload_bw: int
+
+    :param extranet_upload_bw: 外网上传带宽, 单位:Gbps
+    :type extranet_upload_bw: int
+
+    :param total_download_bw: 总下载带宽, 单位:Gbps
+    :type total_download_bw: int
+
+    :param intranet_download_bw: 内外下载带宽, 单位:Gbps
+    :type intranet_download_bw: int
+
+    :param extranet_download_bw: 外网下载带宽, 单位:Gbps
+    :type extranet_download_bw: int
+
+    :param total_qps: 总qps限制
+    :type total_qps: int
+
+    :param intranet_qps: 内网访问qps
+    :type intranet_qps: int
+
+    :param extranet_qps: 外网访问qps
+    :type extranet_qps: int
+    """
+    def __init__(self, 
+            region=None,
+            total_upload_bw = None,
+            intranet_upload_bw = None,
+            extranet_upload_bw = None,
+            total_download_bw = None,
+            intranet_download_bw = None,
+            extranet_download_bw = None,
+            total_qps = None,
+            intranet_qps = None,
+            extranet_qps = None):
+
+        self.region = region
+        self.total_upload_bw = total_upload_bw
+        self.intranet_upload_bw = intranet_upload_bw
+        self.extranet_upload_bw = extranet_upload_bw
+        self.total_download_bw = total_download_bw
+        self.intranet_download_bw = intranet_download_bw
+        self.extranet_download_bw = extranet_download_bw
+        self.total_qps = total_qps
+        self.intranet_qps = intranet_qps
+        self.extranet_qps = extranet_qps
+
+class GetUserQosInfoResult(RequestResult, UserQosInfo):
+    def __init__(self, resp):
+        RequestResult.__init__(self, resp)
+        UserQosInfo.__init__(self)
+
+class GetBucketQosInfoResult(RequestResult, BucketQosInfo):
+    def __init__(self, resp):
+        RequestResult.__init__(self, resp)
+        BucketQosInfo.__init__(self)
+
+class BucketUserQos(object):
+    """用户服务质量。
+    :param int storage_capacity: 容量大小，单位GB
+    """
+    def __init__(self, storage_capacity=None):
+        self.storage_capacity = storage_capacity
+
+class GetBucketUserQosResult(RequestResult, BucketUserQos):
+    def __init__(self, resp):
+        RequestResult.__init__(self, resp)
+        BucketUserQos.__init__(self)
