@@ -47,7 +47,9 @@ from .models import (SimplifiedObjectInfo,
                      REDIRECT_TYPE_MIRROR,
                      REDIRECT_TYPE_EXTERNAL,
                      REDIRECT_TYPE_INTERNAL,
-                     REDIRECT_TYPE_ALICDN)
+                     REDIRECT_TYPE_ALICDN,
+                     NoncurrentVersionStorageTransition,
+                     NoncurrentVersionExpiration)
 
 from .select_params import (SelectJsonTypes, SelectParameters)
 
@@ -213,8 +215,7 @@ def parse_list_parts(result, body):
 
 def parse_batch_delete_objects(result, body):
     if not body:
-        return result
-
+        return result 
     root = ElementTree.fromstring(body)
     url_encoded = _is_url_encoding(root)
 
@@ -294,6 +295,8 @@ def parse_get_bucket_info(result, body):
     result.location = _find_tag(root, 'Bucket/Location')
     result.owner = Owner(_find_tag(root, 'Bucket/Owner/DisplayName'), _find_tag(root, 'Bucket/Owner/ID'))
     result.acl = AccessControlList(_find_tag(root, 'Bucket/AccessControlList/Grant'))
+    result.data_redundancy_type = _find_tag(root, 'Bucket/DataRedundancyType')
+    result.comment = _find_tag(root, 'Bucket/Comment')
 
     server_side_encryption = root.find("Bucket/ServerSideEncryptionRule")
 
@@ -582,6 +585,10 @@ def parse_lifecycle_expiration(expiration_node):
         expiration.days = _find_int(expiration_node, 'Days')
     elif expiration_node.find('Date') is not None:
         expiration.date = iso8601_to_date(_find_tag(expiration_node, 'Date'))
+    elif expiration_node.find('CreatedBeforeDate') is not None:
+        expiration.created_before_date = iso8601_to_date(_find_tag(expiration_node, 'CreatedBeforeDate'))
+    elif expiration_node.find('ExpiredObjectDeleteMarker') is not None:
+        expiration.expired_detete_marker = _find_bool(expiration_node, 'ExpiredObjectDeleteMarker')
 
     return expiration
 
@@ -628,6 +635,25 @@ def parse_lifecycle_object_taggings(lifecycle_tagging_nodes):
 
     return Tagging(tagging_rule)
 
+def parse_lifecycle_version_expiration(version_expiration_node):
+    if version_expiration_node is None:
+        return None
+
+    noncurrent_days = _find_int(version_expiration_node, 'NoncurrentDays')
+    expiration = NoncurrentVersionExpiration(noncurrent_days)
+
+    return expiration
+
+def parse_lifecycle_verison_storage_transitions(version_storage_transition_nodes):
+    version_storage_transitions = []
+    for transition_node in version_storage_transition_nodes:
+        storage_class = _find_tag(transition_node, 'StorageClass')
+        non_crurrent_days = _find_int(transition_node, 'NoncurrentDays')
+        version_storage_transition = NoncurrentVersionStorageTransition(non_crurrent_days, storage_class)
+        version_storage_transitions.append(version_storage_transition)
+
+    return version_storage_transitions
+
 def parse_get_bucket_lifecycle(result, body):
 
     root = ElementTree.fromstring(body)
@@ -638,6 +664,9 @@ def parse_get_bucket_lifecycle(result, body):
         abort_multipart_upload = parse_lifecycle_abort_multipart_upload(rule_node.find('AbortMultipartUpload'))
         storage_transitions = parse_lifecycle_storage_transitions(rule_node.findall('Transition'))
         tagging = parse_lifecycle_object_taggings(rule_node.findall('Tag'))
+        noncurrent_version_expiration = parse_lifecycle_version_expiration(rule_node.find('NoncurrentVersionExpiration'))
+        noncurrent_version_sotrage_transitions = parse_lifecycle_verison_storage_transitions(rule_node.findall('NoncurrentVersionTransition'))
+
         rule = LifecycleRule(
             _find_tag(rule_node, 'ID'),
             _find_tag(rule_node, 'Prefix'),
@@ -645,7 +674,9 @@ def parse_get_bucket_lifecycle(result, body):
             expiration=expiration,
             abort_multipart_upload=abort_multipart_upload,
             storage_transitions=storage_transitions,
-            tagging=tagging 
+            tagging=tagging,
+            noncurrent_version_expiration = noncurrent_version_expiration,
+            noncurrent_version_sotrage_transitions = noncurrent_version_sotrage_transitions
             )
         result.rules.append(rule)
 
@@ -712,7 +743,10 @@ def to_batch_delete_objects_version_request(objectVersions, quiet):
 def to_put_bucket_config(bucket_config):
     root = ElementTree.Element('CreateBucketConfiguration')
 
-    _add_text_child(root, 'StorageClass', str(bucket_config.storage_class))
+    _add_text_child(root, 'StorageClass', bucket_config.storage_class)
+
+    if bucket_config.data_redundancy_type is not None:
+        _add_text_child(root, 'DataRedundancyType', bucket_config.data_redundancy_type)
 
     return _node_to_string(root)
 
@@ -848,6 +882,8 @@ def to_put_bucket_lifecycle(bucket_lifecycle):
                 _add_text_child(expiration_node, 'Date', date_to_iso8601(expiration.date))
             elif expiration.created_before_date is not None:
                 _add_text_child(expiration_node, 'CreatedBeforeDate', date_to_iso8601(expiration.created_before_date))
+            elif expiration.expired_detete_marker is not None:
+                _add_text_child(expiration_node, 'ExpiredObjectDeleteMarker', str(expiration.expired_detete_marker))
 
         abort_multipart_upload = rule.abort_multipart_upload
         if abort_multipart_upload:
@@ -876,6 +912,19 @@ def to_put_bucket_lifecycle(bucket_lifecycle):
                 tag_node = ElementTree.SubElement(rule_node, 'Tag')
                 _add_text_child(tag_node, 'Key', key)
                 _add_text_child(tag_node, 'Value', tagging_rule[key])
+
+        noncurrent_version_expiration = rule.noncurrent_version_expiration
+        if noncurrent_version_expiration is not None:
+            version_expiration_node = ElementTree.SubElement(rule_node, 'NoncurrentVersionExpiration')
+            _add_text_child(version_expiration_node, 'NoncurrentDays', str(noncurrent_version_expiration.noncurrent_days))
+
+        noncurrent_version_sotrage_transitions = rule.noncurrent_version_sotrage_transitions
+        if noncurrent_version_sotrage_transitions is not None:
+            for noncurrent_version_sotrage_transition in noncurrent_version_sotrage_transitions:
+                version_transition_node = ElementTree.SubElement(rule_node, 'NoncurrentVersionTransition')
+                _add_text_child(version_transition_node, 'NoncurrentDays', str(noncurrent_version_sotrage_transition.noncurrent_days))
+                _add_text_child(version_transition_node, 'StorageClass', str(noncurrent_version_sotrage_transition.storage_class))
+
     return _node_to_string(root)
 
 
@@ -976,9 +1025,7 @@ def to_select_json_object(sql, select_params):
     options = ElementTree.SubElement(root, 'Options')
     is_doc = select_params[SelectParameters.Json_Type] == SelectJsonTypes.DOCUMENT
     _add_text_child(json, 'Type', select_params[SelectParameters.Json_Type])
-    if select_params is None:
-        return _node_to_string(root)
-    
+
     for key, value in select_params.items(): 
         if SelectParameters.SplitRange == key and is_doc == False:
             _add_text_child(json, 'Range', utils._make_split_range_string(value))
@@ -1181,3 +1228,63 @@ def parse_get_bucket_request_payment(result, body):
     result.payer = _find_tag(root, 'Payer')
    
     return result
+
+def to_put_qos_info(qos_info):
+    root = ElementTree.Element("QoSConfiguration")
+
+    if qos_info.total_upload_bw is not None:
+        _add_text_child(root, "TotalUploadBandwidth", str(qos_info.total_upload_bw))
+    if qos_info.intranet_upload_bw is not None:
+        _add_text_child(root, "IntranetUploadBandwidth", str(qos_info.intranet_upload_bw))
+    if qos_info.extranet_upload_bw is not None:
+        _add_text_child(root, "ExtranetUploadBandwidth", str(qos_info.extranet_upload_bw))
+    if qos_info.total_download_bw is not None:
+        _add_text_child(root, "TotalDownloadBandwidth", str(qos_info.total_download_bw))
+    if qos_info.intranet_download_bw is not None:
+        _add_text_child(root, "IntranetDownloadBandwidth", str(qos_info.intranet_download_bw))
+    if qos_info.extranet_download_bw is not None:
+        _add_text_child(root, "ExtranetDownloadBandwidth", str(qos_info.extranet_download_bw))
+    if qos_info.total_qps is not None:
+        _add_text_child(root, "TotalQps", str(qos_info.total_qps))
+    if qos_info.intranet_qps is not None:
+        _add_text_child(root, "IntranetQps", str(qos_info.intranet_qps))
+    if qos_info.extranet_qps is not None:
+        _add_text_child(root, "ExtranetQps", str(qos_info.extranet_qps))
+
+    return _node_to_string(root)
+
+def parse_get_qos_info(result, body):
+    """解析UserQosInfo 或者BucketQosInfo
+
+    :UserQosInfo包含成员region,其他成员同BucketQosInfo
+    """
+    root = ElementTree.fromstring(body)
+
+    if hasattr(result, 'region'):
+        result.region = _find_tag(root, 'Region')
+
+    result.total_upload_bw = _find_int(root, 'TotalUploadBandwidth')
+    result.intranet_upload_bw = _find_int(root, 'IntranetUploadBandwidth')
+    result.extranet_upload_bw = _find_int(root, 'ExtranetUploadBandwidth')
+    result.total_download_bw = _find_int(root, 'TotalDownloadBandwidth')
+    result.intranet_download_bw = _find_int(root, 'IntranetDownloadBandwidth')
+    result.extranet_download_bw = _find_int(root, 'ExtranetDownloadBandwidth')
+    result.total_qps = _find_int(root, 'TotalQps')
+    result.intranet_qps = _find_int(root, 'IntranetQps')
+    result.extranet_qps = _find_int(root, 'ExtranetQps')
+
+    return result
+
+def parse_get_bucket_user_qos(result, body):
+    root = ElementTree.fromstring(body)
+
+    result.storage_capacity = _find_int(root, 'StorageCapacity')
+
+    return result
+
+def to_put_bucket_user_qos(user_qos):
+    root = ElementTree.Element('BucketUserQos')
+
+    _add_text_child(root, 'StorageCapacity', str(user_qos.storage_capacity))
+
+    return _node_to_string(root)
